// CycleWnd.cpp : implementation file
//

#include "stdafx.h"
#include <regstr.h>
#include "Cycle.h"
#include "CycleWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////
// Constants for verify password procedure
static const TCHAR g_KszPwdDLL[] = TEXT("PASSWORD.CPL");
static const CHAR  g_KszFnName[] = "VerifyScreenSavePwd"; 

/////////////////////////////////////////////////////////////////////////////
// CCycleWnd
const int CCycleWnd::m_KnInterval = 500;

CCycleWnd::CCycleWnd(): VerifyPasswordProc(NULL),
    m_hPWDDLL(NULL), 
	m_oLastCheckTime(CTime::GetCurrentTime()),
    m_bVerifyingPassword(false),
    m_unEvent( 1 ),
    m_nColorIndex( 0 )
{
}

CCycleWnd::~CCycleWnd()
{
}


BEGIN_MESSAGE_MAP(CCycleWnd, CWnd)
	//{{AFX_MSG_MAP(CCycleWnd)
    ON_WM_SYSCOMMAND()
    ON_WM_SETCURSOR()
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONDOWN()
    ON_WM_MBUTTONDOWN()
    ON_WM_RBUTTONDOWN()
    ON_WM_KEYDOWN()
    ON_WM_SYSKEYDOWN()
	ON_WM_PAINT()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CCycleWnd message handlers
void CCycleWnd::PostNcDestroy() 
{
    if (NULL != m_hPWDDLL) 
    {
        FreeLibrary(m_hPWDDLL);
        m_hPWDDLL = NULL;
    }
    delete this;
}

void CCycleWnd::OnSysCommand(UINT nID, LPARAM lParam) 
{
    if ((SC_SCREENSAVE == nID) ||
         (SC_PREVWINDOW == nID) ||
         (SC_NEXTWINDOW == nID)) 
    { // eat the message
    }
    else 
    {
        CWnd::OnSysCommand(nID, lParam);
    }
}

BOOL CCycleWnd::OnSetCursor(CWnd* pWnd, UINT nHitTest, 
                                  UINT message) 
{
    if (NULL == GetParent()) 
    {
        if ( false == m_bVerifyingPassword )
        {
            SetCursor(NULL);
        }
        else
        {
            CWnd::OnSetCursor( pWnd, nHitTest, message );
        }
    }
	return TRUE;
}

void CCycleWnd::OnMouseMove(UINT nFlags, CPoint point)
{
    CWnd::OnMouseMove(nFlags, point);
    CheckClosing();
}

void CCycleWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
    CWnd::OnLButtonDown(nFlags, point);
    CheckClosing();
}

void CCycleWnd::OnMButtonDown(UINT nFlags, CPoint point) 
{
    CWnd::OnMButtonDown(nFlags, point);
    CheckClosing();
}

void CCycleWnd::OnRButtonDown(UINT nFlags, CPoint point) 
{
    CWnd::OnRButtonDown(nFlags, point);
    CheckClosing();
}

void CCycleWnd::OnKeyDown(UINT nChar, UINT nRepCnt, 
                                UINT nFlags) 
{
    CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
    CheckClosing();
}

void CCycleWnd::OnSysKeyDown(UINT nChar, UINT nRepCnt, 
                                   UINT nFlags) 
{
    CWnd::OnSysKeyDown(nChar, nRepCnt, nFlags);
    CheckClosing();
}

BOOL CCycleWnd::Create(HWND hParentWnd/*= NULL*/)
{
    DWORD dwStyle = 0;
    DWORD dwExStyle = 0;
    CRect oRect;

    if (NULL == hParentWnd) 
    { // Setup options for screensaver mode
        dwStyle = WS_POPUP | WS_VISIBLE;
        dwExStyle = WS_EX_TOPMOST;
        oRect = CRect(0, 0, GetSystemMetrics( SM_CXSCREEN), 
            GetSystemMetrics(SM_CYSCREEN ));
    }
    else 
    { // Setup options for preview mode
        dwStyle = WS_VISIBLE | WS_CHILD;
        dwExStyle = NULL;
		CWnd* pParent = CWnd::FromHandle(hParentWnd);
        // If the window is invalid, leave.
        if ((NULL == pParent) || (!::IsWindow(hParentWnd)))
        { 
            return FALSE;
        }
        pParent->GetClientRect(&oRect);
    }
   	CString szClassName = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW
        |CS_SAVEBITS|CS_DBLCLKS, NULL);

    // look in registry to see if password protection is turned on,
    // otherwise don't bother to load the password handler DLL
    HKEY hKey;
    if (NULL == hParentWnd) 
    { // Don't do this unless we are in screen saver mode
        if (ERROR_SUCCESS == RegOpenKey(HKEY_CURRENT_USER,
            REGSTR_PATH_SCREENSAVE,&hKey)) 
        {
            DWORD dwVal = 0;
            DWORD dwSize=sizeof(dwVal);
            if ((RegQueryValueEx(hKey,REGSTR_VALUE_USESCRPASSWORD, 
                NULL,NULL,(BYTE *) &dwVal,&dwSize) == ERROR_SUCCESS)
                && dwVal) 
            {
                // try to load the DLL that contains password proc.
                m_hPWDDLL = LoadLibrary(g_KszPwdDLL);
                if (m_hPWDDLL) 
                {  // Get the address of the procedure
                    VerifyPasswordProc = (VERIFYPWDPROC) 
                        GetProcAddress(m_hPWDDLL, g_KszFnName);
                    if( !VerifyPasswordProc) 
                    {
                        FreeLibrary(m_hPWDDLL);
                        m_hPWDDLL = NULL;
                    }
                }
            }
            RegCloseKey(hKey);
        }
    }
    // Lastly, create the screen saver window.
    BOOL bRetVal = CreateEx(dwExStyle, szClassName, _T(""), dwStyle, 
        oRect.left, oRect.top, oRect.Width(), 
        oRect.Height(), hParentWnd, NULL, NULL);

    m_unEvent = SetTimer( m_unEvent, m_KnInterval, NULL );
    return bRetVal;
}

bool CCycleWnd::VerifyPassword()
{
    bool bRetVal = false;
    m_bVerifyingPassword = true;
    if (NULL != VerifyPasswordProc) 
    {
        bRetVal = (VerifyPasswordProc(GetSafeHwnd()) != 0);
    }
    else if ( NULL == GetParent() )
    {
        bRetVal = true;
    }
    m_bVerifyingPassword = false;
    return bRetVal;
}

void CCycleWnd::CheckClosing()
{
    // Don't verify less than 1s after the last check
    CTime oNow = CTime::GetCurrentTime();
    static const CTimeSpan SKoTimeSpan(0, 0, 0, 1);
    if ((oNow - m_oLastCheckTime) > SKoTimeSpan) 
    {
        if (VerifyPassword()) 
        {
            PostMessage(WM_CLOSE, 0, 0);
        }
        m_oLastCheckTime = CTime::GetCurrentTime();
    }
}

const COLORREF g_Kcolors[] = {
    RGB( 0x0, 0x0, 0x0 ),
    RGB( 0xff, 0x0, 0x0 ),
    RGB( 0x0, 0xff, 0x0 ),
    RGB( 0x0, 0x0, 0xff )
};

void CCycleWnd::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CRect oRect;
	CBrush oBrush(g_Kcolors[m_nColorIndex]);
	GetClientRect(oRect);
	dc.FillRect(oRect, &oBrush);
    ++m_nColorIndex;
    if ( m_nColorIndex >= 4 )
    {
        m_nColorIndex = 0;
    }
}

void CCycleWnd::OnTimer(UINT nIDEvent) 
{
    if ( nIDEvent == m_unEvent )
    {
        KillTimer( m_unEvent );
        Invalidate();
        m_unEvent = SetTimer( m_unEvent, m_KnInterval, NULL );
    }
	CWnd::OnTimer(nIDEvent);
}
