// InvrtWnd.cpp : implementation file
//

#include "stdafx.h"
#include <regstr.h>
#include "Invrt.h"
#include "InvrtWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////
// Constants for verify password procedure
static const TCHAR g_KszPwdDLL[] = TEXT("PASSWORD.CPL");
static const CHAR  g_KszFnName[] = "VerifyScreenSavePwd"; 

/////////////////////////////////////////////////////////////////////////////
// CInvrtWnd

const int CInvrtWnd::m_KnInterval = 1;

CInvrtWnd::CInvrtWnd(): VerifyPasswordProc(NULL),
    m_hPWDDLL(NULL), 
	m_oLastCheckTime(CTime::GetCurrentTime()),
    m_bVerifyingPassword(false),
    m_unEvent( 1 ),
    m_nCurrentLine( 0 ),
    m_eSwapDirection( HORIZONTAL )
{
}

CInvrtWnd::~CInvrtWnd()
{
}


BEGIN_MESSAGE_MAP(CInvrtWnd, CWnd)
	//{{AFX_MSG_MAP(CInvrtWnd)
    ON_WM_SYSCOMMAND()
    ON_WM_SETCURSOR()
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONDOWN()
    ON_WM_MBUTTONDOWN()
    ON_WM_RBUTTONDOWN()
    ON_WM_KEYDOWN()
    ON_WM_SYSKEYDOWN()
	ON_WM_TIMER()
	ON_WM_ERASEBKGND()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CInvrtWnd message handlers
void CInvrtWnd::PostNcDestroy() 
{
    if (NULL != m_hPWDDLL) 
    {
        FreeLibrary(m_hPWDDLL);
        m_hPWDDLL = NULL;
    }
    delete this;
}

void CInvrtWnd::OnSysCommand(UINT nID, LPARAM lParam) 
{
    if ((SC_SCREENSAVE == nID) ||
         (SC_PREVWINDOW == nID) ||
         (SC_NEXTWINDOW == nID)) 
    { // eat the message
    }
    else 
    {
        CWnd::OnSysCommand(nID, lParam);
    }
}

BOOL CInvrtWnd::OnSetCursor(CWnd* pWnd, UINT nHitTest, 
                                  UINT message) 
{
    if (NULL == GetParent()) 
    {
        if ( false == m_bVerifyingPassword )
        {
            SetCursor(NULL);
        }
        else
        {
            CWnd::OnSetCursor( pWnd, nHitTest, message );
        }
    }
	return TRUE;
}

void CInvrtWnd::OnMouseMove(UINT nFlags, CPoint point)
{
    CWnd::OnMouseMove(nFlags, point);
    CheckClosing();
}

void CInvrtWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
    CWnd::OnLButtonDown(nFlags, point);
    CheckClosing();
}

void CInvrtWnd::OnMButtonDown(UINT nFlags, CPoint point) 
{
    CWnd::OnMButtonDown(nFlags, point);
    CheckClosing();
}

void CInvrtWnd::OnRButtonDown(UINT nFlags, CPoint point) 
{
    CWnd::OnRButtonDown(nFlags, point);
    CheckClosing();
}

void CInvrtWnd::OnKeyDown(UINT nChar, UINT nRepCnt, 
                                UINT nFlags) 
{
    CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
    CheckClosing();
}

void CInvrtWnd::OnSysKeyDown(UINT nChar, UINT nRepCnt, 
                                   UINT nFlags) 
{
    CWnd::OnSysKeyDown(nChar, nRepCnt, nFlags);
    CheckClosing();
}

BOOL CInvrtWnd::Create(HWND hParentWnd/*= NULL*/)
{
    DWORD dwStyle = 0;
    DWORD dwExStyle = 0;
    CRect oRect;

    if (NULL == hParentWnd) 
    { // Setup options for screensaver mode
        dwStyle = WS_POPUP | WS_VISIBLE;
        dwExStyle = WS_EX_TOPMOST;
        oRect = CRect(0, 0, GetSystemMetrics( SM_CXSCREEN), 
            GetSystemMetrics(SM_CYSCREEN ));
    }
    else 
    { // Setup options for preview mode
        dwStyle = WS_VISIBLE | WS_CHILD;
        dwExStyle = NULL;
		CWnd* pParent = CWnd::FromHandle(hParentWnd);
        // If the window is invalid, leave.
        if ((NULL == pParent) || (!::IsWindow(hParentWnd)))
        { 
            return FALSE;
        }
        pParent->GetClientRect(&oRect);
    }
   	CString szClassName = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW
        |CS_SAVEBITS|CS_DBLCLKS, NULL);

    // look in registry to see if password protection is turned on,
    // otherwise don't bother to load the password handler DLL
    HKEY hKey;
    if (NULL == hParentWnd) 
    { // Don't do this unless we are in screen saver mode
        if (ERROR_SUCCESS == RegOpenKey(HKEY_CURRENT_USER,
            REGSTR_PATH_SCREENSAVE,&hKey)) 
        {
            DWORD dwVal = 0;
            DWORD dwSize=sizeof(dwVal);
            if ((RegQueryValueEx(hKey,REGSTR_VALUE_USESCRPASSWORD, 
                NULL,NULL,(BYTE *) &dwVal,&dwSize) == ERROR_SUCCESS)
                && dwVal) 
            {
                // try to load the DLL that contains password proc.
                m_hPWDDLL = LoadLibrary(g_KszPwdDLL);
                if (m_hPWDDLL) 
                {  // Get the address of the procedure
                    VerifyPasswordProc = (VERIFYPWDPROC) 
                        GetProcAddress(m_hPWDDLL, g_KszFnName);
                    if( !VerifyPasswordProc) 
                    {
                        FreeLibrary(m_hPWDDLL);
                        m_hPWDDLL = NULL;
                    }
                }
            }
            RegCloseKey(hKey);
        }
    }
    // Lastly, create the screen saver window.
    BOOL bRetVal = CreateEx(dwExStyle, szClassName, _T(""), dwStyle, 
        oRect.left, oRect.top, oRect.Width(), 
        oRect.Height(), hParentWnd, NULL, NULL);
    m_unEvent = SetTimer( m_unEvent, m_KnInterval, NULL );
    return bRetVal;
}

bool CInvrtWnd::VerifyPassword()
{
    bool bRetVal = false;
    m_bVerifyingPassword = true;
    if (NULL != VerifyPasswordProc) 
    {
        bRetVal = (VerifyPasswordProc(GetSafeHwnd()) != 0);
        Invalidate();
    }
    else if ( NULL == GetParent() )
    {
        bRetVal = true;
    }
    m_bVerifyingPassword = false;
    return bRetVal;
}

void CInvrtWnd::CheckClosing()
{
    // Don't verify less than 1s after the last check
    CTime oNow = CTime::GetCurrentTime();
    static const CTimeSpan SKoTimeSpan(0, 0, 0, 1);
    if ((oNow - m_oLastCheckTime) > SKoTimeSpan) 
    {
        if (VerifyPassword()) 
        {
            KillTimer( m_unEvent );
            PostMessage(WM_CLOSE, 0, 0);
        }
        m_oLastCheckTime = CTime::GetCurrentTime();
    }
}

void CInvrtWnd::OnTimer(UINT nIDEvent) 
{
    if ( nIDEvent == m_unEvent )
    {
        KillTimer( m_unEvent );
        SwapLines();
        m_unEvent = SetTimer( m_unEvent, m_KnInterval, NULL );
    }
	CWnd::OnTimer(nIDEvent);
}

void CInvrtWnd::SwapLines()
{
	CWindowDC dc(this); // device context for painting
    CRect oRect;
    GetClientRect( oRect );
    int nWidth = oRect.Width();
    int nHeight = oRect.Height();
    if ( HORIZONTAL == m_eSwapDirection )
    {
        COLORREF colorTop;
        COLORREF colorBottom;
        int nTopLine = m_nCurrentLine;
        int nBottomLine = nHeight - m_nCurrentLine;
        for ( int i = 0; i < nWidth; ++i )
        {
            colorTop = dc.GetPixel( i, nTopLine );
            colorBottom = dc.GetPixel( i, nBottomLine );
            dc.SetPixel( i, nTopLine, colorBottom );
            dc.SetPixel( i, nBottomLine, colorTop );
        }
    }
    else
    {
        COLORREF colorLeft;
        COLORREF colorRight;
        int nLeftLine = m_nCurrentLine;
        int nRightLine = nWidth - m_nCurrentLine;
        for ( int i = 0; i < nHeight; ++i )
        {
            colorLeft = dc.GetPixel( nLeftLine, i );
            colorRight = dc.GetPixel( nRightLine, i );
            dc.SetPixel( nLeftLine, i, colorRight );
            dc.SetPixel( nRightLine, i, colorLeft );
        }
    }
    if ( m_nCurrentLine >= ( ( HORIZONTAL == m_eSwapDirection ) ? nHeight/2 : nWidth/2 ) )
    {
        m_nCurrentLine = -1;
        if ( HORIZONTAL == m_eSwapDirection )
        {
            m_eSwapDirection = VERTICAL;
        }
        else
        {
            m_eSwapDirection = HORIZONTAL;
        }
    }
    ++m_nCurrentLine;
}


BOOL CInvrtWnd::OnEraseBkgnd(CDC* pDC) 
{
    CBitmap oBitmap;
    oBitmap.LoadBitmap( IDB_SS );
    CDC dc;
    dc.CreateCompatibleDC( pDC );
    dc.SelectObject( oBitmap );
    BITMAP bm;
    oBitmap.GetBitmap( &bm );
    CRect oRect;
    GetClientRect( oRect );
    pDC->StretchBlt( oRect.TopLeft().x, oRect.TopLeft().y, oRect.Width(), oRect.Height(),
        &dc, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY );
    pDC->SetBkColor( RGB( 0, 0, 0 ) );
    m_nCurrentLine = 0;
    m_eSwapDirection = HORIZONTAL;
    return TRUE;
}

void CInvrtWnd::OnDestroy() 
{
    KillTimer( m_unEvent );
	CWnd::OnDestroy();
}
