// WDJWnd.cpp : implementation file
//

#include "stdafx.h"
#include <regstr.h>
#include "WDJ.h"
#include "WDJWnd.h"
#include "cdib.h"
#include <vector>
#if !defined(AFX_WDJSETTINGS_H__0066CB20_56E2_11D2_A59B_00C04F94E929__INCLUDED_)
    #include "wdjset.h"
#endif //if !defined(AFX_WDJSETTINGS_H__0066CB20_56E2_11D2_A59B_00C04F94E929__INCLUDED_)
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////
// Constants for verify password procedure
static const TCHAR g_KszPwdDLL[] = TEXT("PASSWORD.CPL");
static const CHAR  g_KszFnName[] = "VerifyScreenSavePwd"; 

/////////////////////////////////////////////////////////////////////////////
// CWDJWnd

CWDJWnd::CWDJWnd(): VerifyPasswordProc(NULL),
    m_hPWDDLL(NULL), 
	m_oLastCheckTime(CTime::GetCurrentTime()),
    m_bVerifyingPassword(false),
    m_nGridDimension( 1 ),
    m_unEvent( 1 ),
    m_eDirection( Increasing )
{
    CWDJSettings settings;
    m_nInterval = settings.GetDelay();
}

CWDJWnd::~CWDJWnd()
{
}


BEGIN_MESSAGE_MAP(CWDJWnd, CWnd)
	//{{AFX_MSG_MAP(CWDJWnd)
    ON_WM_SYSCOMMAND()
    ON_WM_SETCURSOR()
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONDOWN()
    ON_WM_MBUTTONDOWN()
    ON_WM_RBUTTONDOWN()
    ON_WM_KEYDOWN()
    ON_WM_SYSKEYDOWN()
	ON_WM_ERASEBKGND()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CWDJWnd message handlers
void CWDJWnd::PostNcDestroy() 
{
    if (NULL != m_hPWDDLL) 
    {
        FreeLibrary(m_hPWDDLL);
        m_hPWDDLL = NULL;
    }
    delete this;
}

void CWDJWnd::OnSysCommand(UINT nID, LPARAM lParam) 
{
    if ((SC_SCREENSAVE == nID) ||
         (SC_PREVWINDOW == nID) ||
         (SC_NEXTWINDOW == nID)) 
    { // eat the message
    }
    else 
    {
        CWnd::OnSysCommand(nID, lParam);
    }
}

BOOL CWDJWnd::OnSetCursor(CWnd* pWnd, UINT nHitTest, 
                                  UINT message) 
{
    if (NULL == GetParent()) 
    {
        if ( false == m_bVerifyingPassword )
        {
            SetCursor(NULL);
        }
        else
        {
            CWnd::OnSetCursor( pWnd, nHitTest, message );
        }
    }
	return TRUE;
}

void CWDJWnd::OnMouseMove(UINT nFlags, CPoint point)
{
    CWnd::OnMouseMove(nFlags, point);
    CheckClosing();
}

void CWDJWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
    CWnd::OnLButtonDown(nFlags, point);
    CheckClosing();
}

void CWDJWnd::OnMButtonDown(UINT nFlags, CPoint point) 
{
    CWnd::OnMButtonDown(nFlags, point);
    CheckClosing();
}

void CWDJWnd::OnRButtonDown(UINT nFlags, CPoint point) 
{
    CWnd::OnRButtonDown(nFlags, point);
    CheckClosing();
}

void CWDJWnd::OnKeyDown(UINT nChar, UINT nRepCnt, 
                                UINT nFlags) 
{
    CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
    CheckClosing();
}

void CWDJWnd::OnSysKeyDown(UINT nChar, UINT nRepCnt, 
                                   UINT nFlags) 
{
    CWnd::OnSysKeyDown(nChar, nRepCnt, nFlags);
    CheckClosing();
}

BOOL CWDJWnd::Create(HWND hParentWnd/*= NULL*/)
{
    DWORD dwStyle = 0;
    DWORD dwExStyle = 0;
    CRect oRect;

    if (NULL == hParentWnd) 
    { // Setup options for screensaver mode
        dwStyle = WS_POPUP | WS_VISIBLE;
        dwExStyle = WS_EX_TOPMOST;
        oRect = CRect(0, 0, GetSystemMetrics( SM_CXSCREEN), 
            GetSystemMetrics(SM_CYSCREEN ));
    }
    else 
    { // Setup options for preview mode
        dwStyle = WS_VISIBLE | WS_CHILD;
        dwExStyle = NULL;
		CWnd* pParent = CWnd::FromHandle(hParentWnd);
        // If the window is invalid, leave.
        if ((NULL == pParent) || (!::IsWindow(hParentWnd)))
        { 
            return FALSE;
        }
        pParent->GetClientRect(&oRect);
    }
   	CString szClassName = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW
        |CS_SAVEBITS|CS_DBLCLKS, NULL);

    // look in registry to see if password protection is turned on,
    // otherwise don't bother to load the password handler DLL
    HKEY hKey;
    if (NULL == hParentWnd) 
    { // Don't do this unless we are in screen saver mode
        if (ERROR_SUCCESS == RegOpenKey(HKEY_CURRENT_USER,
            REGSTR_PATH_SCREENSAVE,&hKey)) 
        {
            DWORD dwVal = 0;
            DWORD dwSize=sizeof(dwVal);
            if ((RegQueryValueEx(hKey,REGSTR_VALUE_USESCRPASSWORD, 
                NULL,NULL,(BYTE *) &dwVal,&dwSize) == ERROR_SUCCESS)
                && dwVal) 
            {
                // try to load the DLL that contains password proc.
                m_hPWDDLL = LoadLibrary(g_KszPwdDLL);
                if (m_hPWDDLL) 
                {  // Get the address of the procedure
                    VerifyPasswordProc = (VERIFYPWDPROC) 
                        GetProcAddress(m_hPWDDLL, g_KszFnName);
                    if( !VerifyPasswordProc) 
                    {
                        FreeLibrary(m_hPWDDLL);
                        m_hPWDDLL = NULL;
                    }
                }
            }
            RegCloseKey(hKey);
        }
    }
    // Lastly, create the screen saver window.
    BOOL bRetVal = CreateEx(dwExStyle, szClassName, _T(""), dwStyle, 
        oRect.left, oRect.top, oRect.Width(), 
        oRect.Height(), hParentWnd, NULL, NULL);

    m_unEvent = SetTimer( m_unEvent, m_nInterval, NULL );
    return bRetVal;
}

bool CWDJWnd::VerifyPassword()
{
    bool bRetVal = false;
    m_bVerifyingPassword = true;
    if (NULL != VerifyPasswordProc) 
    {
        bRetVal = (VerifyPasswordProc(GetSafeHwnd()) != 0);
    }
    else if ( NULL == GetParent() )
    {
        bRetVal = true;
    }
    m_bVerifyingPassword = false;
    return bRetVal;
}

void CWDJWnd::CheckClosing()
{
    // Don't verify less than 1s after the last check
    CTime oNow = CTime::GetCurrentTime();
    static const CTimeSpan SKoTimeSpan(0, 0, 0, 1);
    if ((oNow - m_oLastCheckTime) > SKoTimeSpan) 
    {
        if (VerifyPassword()) 
        {
            PostMessage(WM_CLOSE, 0, 0);
        }
        m_oLastCheckTime = CTime::GetCurrentTime();
    }
}


BOOL CWDJWnd::OnEraseBkgnd(CDC* pDC) 
{
    using std::vector;
    CWDJSettings settings;
    CDib dib;
    dib.LoadBitmap( IDB_WDJ );
    CRect oRect;
    vector<CPoint> vPoint;
    GetClientRect( oRect );
    dib.UsePalette( pDC );
    int nWidth = oRect.Width() / m_nGridDimension;
    int nHeight = oRect.Height() / m_nGridDimension;
    CSize oSize( nWidth, nHeight );
    for ( int i = 0; i < m_nGridDimension; ++i )
    {
        for ( int j = 0; j < m_nGridDimension; ++j )
        {
            CPoint oTempPoint( j * nWidth, i * nHeight );
            vPoint.insert( vPoint.end(), oTempPoint );
        }
    }
    int nVectorSize = vPoint.size();
    for ( int k = 0; k < nVectorSize; ++k )
    {
        dib.Draw( pDC, vPoint[k], oSize );
    }
    if ( m_eDirection == Increasing )
    {
        ++m_nGridDimension;
    }
    else
    {
        --m_nGridDimension;
    }

    if ( m_nGridDimension >= settings.GetRowCount() )
    {
        m_eDirection = Decreasing;
    }
    else if ( m_nGridDimension <= 1 )
    {
        m_eDirection = Increasing;
    }

	return CWnd::OnEraseBkgnd(pDC);
}

void CWDJWnd::OnTimer(UINT nIDEvent) 
{
    if ( m_unEvent == nIDEvent )
    {
        KillTimer( m_unEvent );
        Invalidate();
        m_unEvent = SetTimer( m_unEvent, m_nInterval, NULL );
    }
	CWnd::OnTimer(nIDEvent);
}

void CWDJWnd::OnDestroy() 
{
    KillTimer( m_unEvent );
    CWnd::OnDestroy();
}
